import discord
from v6d2ctx.context import Context, Explicit

from v6d3music.core.mainaudio import MainAudio
from v6d3music.core.queueaudio import QueueAudio

mainasrcs: dict[discord.Guild, MainAudio] = {}


async def raw_vc_for(ctx: Context) -> discord.VoiceClient:
    if ctx.guild is None:
        raise Explicit('not in a guild')
    vc: discord.VoiceProtocol = ctx.guild.voice_client
    if vc is None or isinstance(vc, discord.VoiceClient) and not vc.is_connected():
        vs: discord.VoiceState = ctx.member.voice
        if vs is None:
            raise Explicit('not connected')
        vch: discord.VoiceChannel = vs.channel
        if vch is None:
            raise Explicit('not connected')
        try:
            vc: discord.VoiceProtocol = await vch.connect()
        except discord.ClientException:
            vc: discord.VoiceProtocol = ctx.guild.voice_client
            await ctx.guild.fetch_channels()
            await vc.disconnect(force=True)
            raise Explicit('try again later')
    assert isinstance(vc, discord.VoiceClient)
    return vc


async def main_for_raw_vc(vc: discord.VoiceClient, *, create: bool, force_play: bool) -> MainAudio:
    if vc.guild in mainasrcs:
        source = mainasrcs[vc.guild]
    else:
        if create:
            source = mainasrcs.setdefault(
                vc.guild,
                await MainAudio.create(vc.guild)
            )
        else:
            raise Explicit('not playing, use `queue pause` or `queue resume`')
    if vc.source != source or create and not vc.is_playing() and (force_play or not vc.is_paused()):
        vc.play(source)
    return source


async def vc_main_for(ctx: Context, *, create: bool, force_play: bool) -> tuple[discord.VoiceClient, MainAudio]:
    vc = await raw_vc_for(ctx)
    return vc, await main_for_raw_vc(vc, create=create, force_play=force_play)


async def vc_for(ctx: Context, *, create: bool, force_play: bool) -> discord.VoiceClient:
    vc, source = await vc_main_for(ctx, create=create, force_play=force_play)
    return vc


async def main_for(ctx: Context, *, create: bool, force_play: bool) -> MainAudio:
    vc, source = await vc_main_for(ctx, create=create, force_play=force_play)
    return source


async def queue_for(ctx: Context, *, create: bool, force_play: bool) -> QueueAudio:
    return (await main_for(ctx, create=create, force_play=force_play)).queue
